// Extremely simple program to demonstrate the ZIP writer class
// Public domain

#include <cstdio>
#include <string>
#include <vector>

#include <zipwriter.h>

using namespace std;

static void usage()
{
    printf("Usage: testzipwrite <zip file> <file list>\n");
    printf("The file list is a file where each line specifies one directory or a file\n");
    printf("(relative to the current directory) that you want to put in the ZIP. The\n");
    printf("list file is always UTF-8 encoded, even under Windows.\n");
    printf("Each line in the list file begins with either 'D' (for directory) or 'F' (for\n");
    printf("file). After that comes some whitespace, followed by the name.\n");
}

static void trim(string &s)
{
    static const char *ws = " \t\n\r";

    s.erase(s.find_last_not_of(ws) + 1);
    s.erase(0, s.find_first_not_of(ws));
}

int main(int argc, char *argv[])
{
    ZipWriter zip;

    if (argc != 3) {
        usage();
        return 1;
    }

    // open the ZIP
    if (!zip.open(argv[1])) {
        printf("Can't open the ZIP: %s (%d)\n", zip.errorString(), zip.getError());
        return 1;
    }

    // load the file list
    FILE *fl = fopen(argv[2], "rt");

    if (!fl) {
        printf("Can't open the list file\n");
        return 1;
    }

    printf("Creating archive %s\n", argv[1]);

    zip.setArchiveComment("Created with testzipwriter to demonstrate the ZipWriter class.");

    while (true) {
        char buffer[1024];

        fgets(buffer, 1024, fl);

        if (feof(fl))
            break;

        string s(buffer);

        trim(s);

        if (s.empty())
            continue;

        string name = s.substr(1);

        trim(name);

        bool ret;

        if (s[0] == 'D')
            ret = zip.addDirectory(name);
        else if (s[0] == 'F')
            ret = zip.addFile(name);
        else {
            printf("Unknown type '%c', ignored\n", s[0]);
            continue;
        }

        if (!ret) {
            printf("%s: %s (%d)\n", name.c_str(), zip.errorString(), zip.getError());
            fclose(fl);
            zip.cancel();
            return 1;
        } else printf("Added %s %s\n", s[0] == 'D' ? "directory" : "file", name.c_str());
    }

    fclose(fl);

    if (!zip.finish()) {
        printf("Could not finish the archive: %s (%d)\n", zip.errorString(), zip.getError());
        return 1;
    }

    return 0;
}
